@@ -19,7 +19,9 @@ summary.gts <- function(object, ...) {
     cat(paste("Forecast method:", fmethod), "\n")
     if (!is.null(object$fitted)) {
       cat("In-sample error measures at the bottom level: \n")
-      print(accuracy.gts(object))
+      print(accuracy.gts(object, 
+                         levels = ifelse(is.hts(object), length(object$nodes),
+                                         nrow(object$groups) - 1L)))
     }
   }
 }
